#!/usr/bin/python
#
# calc_ang
# Program to calculate the angular distance between two onjects in the sky
# input: (coords1, coords2) : ([RA, Dec1], [RA2, Dec2]) : 
#  ([hrs1, min1, sec1, deg1, arcmin1, arcsec1], [hrs2, min2, sec2, deg2, arcmin2, arcsec2])
# The input is two lists with each list containing exactly 6 numbers corresponding to
# the RA and Dec coordinates of the two targets.
# e.g. calc_ang([18,23,32.11,-30,00,00.0],[15,12,00,2,00,00])
# output: angle (degrees, arcmin, arcsec)
#
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Feb 2009
# Last modification by Y.Tsapras 13 Apr 2011 - added code to catch errors
####################################################################################

# Import required modules
from math import cos, sin, acos, degrees, radians
from hms2dh import hms2dh
from dh2hms import dh2hms
from dms2dd import dms2dd
from dd2dms import dd2dms
from sys import argv, exit


####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


####################################################################################
# Function check_inp
# Input is a list of six numbers = [xx,xx,xx,yy,yy,yy] where xx correspond to the RA of
# a target and yy to the Dec.
# Check that the input is valid and exit with an error message if it isn't.
def check_inp(coords):
    # Define a new array that will hold the coordinates in the correct format
    coords_ok = []
    # Raise an error if there are fewer than or more than 6 numbers for the input list
    if (len(coords) != 6): 
        print "Error: Input must be a list of 6 numbers."
	exit()
    # Check that they are all valid numbers
    for num in coords:
        #print "num =", num # uncomment for debugging
	#print is_number(num) # uncomment for debugging
        if (is_number(num) == False):
	    print "Error: %s is not a valid number." % str(num)
	    exit()
	else:
	    # ensure that all elements are numbers
	    coords_ok.append( float(num) )
    
    # Check that the RA is between 0 and 24 hrs otherwise return an error
    if ( float(coords_ok[0]) > 24 or float( coords_ok[0] ) < 0 ):
        print "RA cannot be negative of greater than 24." 
        exit()
    # Return the coordinates in the correct format
    #print "coords_ok=", coords_ok  # uncomment for debugging
    return coords_ok


####################################################################################
# Function calc_ang
# The input is two lists with each list containing exactly 6 numbers corresponding to
# the RA and Dec coordinates of the two targets.
# e.g. calc_ang([18,23,32.11,-30,00,00.0],[15,12,00,2,00,00])
# Will canculate the angular distance between the two astronomical targets
def calc_ang(coords_1,coords_2):
    # Check input values
    coords_1, coords_2  = check_inp(coords_1), check_inp(coords_2)
    
    # Print the input coordinates on screen 
    print 'You have given the following coordinates:'
    print 'Object 1 : RA --> %d %d %.2f' % (coords_1[0], coords_1[1], coords_1[2])
    print 'Object 1 : Dec --> %d %d %.2f' % (coords_1[3], coords_1[4], coords_1[5]) 
    print 'Object 2 : RA --> %d %d %.2f' % (coords_2[0], coords_2[1], coords_2[2])
    print 'Object 2 : Dec --> %d %d %.2f' % (coords_2[3], coords_2[4], coords_2[5])     

    # Convert both sets of coordinates to decimal form
    c1_RA = hms2dh(coords_1[0], coords_1[1], coords_1[2]) # in hours
    c1_Dec = dms2dd(coords_1[3], coords_1[4], coords_1[5]) # in degrees
    c2_RA = hms2dh(coords_2[0], coords_2[1], coords_2[2]) # in hours
    c2_Dec = dms2dd(coords_2[3], coords_2[4], coords_2[5]) # in degrees

    # Convert Declination angles from degrees to radians
    cr1_Dec = radians(c1_Dec)
    cr2_Dec = radians(c2_Dec)

    # Calculate RA1 - RA2 and convert to degrees by multiplying by 15
    diff_degrees = (c1_RA - c2_RA) * 15  # in degrees
    diff_radians = radians(diff_degrees) # in radians

    cos_angle_radians = (sin(cr1_Dec) * sin(cr2_Dec) + \
                         cos(cr1_Dec) * cos(cr2_Dec) * \
                         cos(diff_radians)) # All values in radians
    
    angle_radians = acos(cos_angle_radians)
    angle_degrees = degrees(angle_radians)
    angle_dms = dd2dms(angle_degrees)
    
    print 'Sky separation angle = %d degrees %d arcmin %.2f arcsec' % (angle_dms[0],angle_dms[1],angle_dms[2])
    
    return angle_dms


####################################################################################
# Commandline run section
# If given 12 space separated numbers from the command line, call calc_ang and return the angular distance.
# The 12 numbers are the RA and Dec for objects 1 and 2.
# e.g. python calc_ang.py 18 23 32.11 -30 00 00.0 15 12 10.01 02 00 12.34
if __name__ == '__main__':
    if (len(argv) != 13): 
        print "Error: Incorrect number of arguments. Need 12 numbers for RA and Dec of both targets."
	print "Example calling sequence: python calc_ang.py 18 23 32.11 -30 00 00.0 15 12 10.01 02 00 12.34"
	exit()
        
    # Check input and set values
    coords_1, coords_2  = check_inp([i for i in argv[1:7]]), check_inp([i for i in argv[7:]])
    
    # Set values    
    #coords_1, coords_2 = [ float(i) for i in argv[1:7] ], [ float(i) for i in argv[7:] ]
    #print coords_1, coords_2 # uncomment for debugging
    calc_ang(coords_1,coords_2)
    
