#!/usr/bin/python
#
# ecl2equa
# Program to convert ecliptic to equatorial coordinates
# input: ecliptic longitude, ecliptic latitude (in dms), obliquity
# output: RA, Dec in decimal hours, degrees
# "obliquity" is the obliquity of the ecliptic, the angle between the planes of
#  equator and the ecliptic (in decimal degrees). 
#
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Feb 2009
# Modification history: 
# - Y. Tsapras 21 May 2010 - version 1.0
# - Y. Tsapras 19 Apr 2011 - added code to catch errors
# - Y. Tsapras 19 Apr 2011 - added commandline run section
########################################################################################

# Import required libraries
from math import cos, sin, tan, asin, acos, atan
from math import degrees
from math import radians
from dms2dd import dms2dd
from dd2dms import dd2dms
from dh2hms import dh2hms
from sys import argv, exit

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


########################################################################################
# Function ecl2equa starts here
# converts ecliptic to equatorial coordinates
def ecl2equa(deg_long, min_long, sec_long, deg_lat, min_lat, sec_lat, obliquity):
    # check that the input values are all numbers:
    for i in [deg_long, min_long, sec_long, deg_lat, min_lat, sec_lat, obliquity]:
        if ( is_number(i) == False ):
	    print "Error: %s is not a valid number." % str(i)
	    exit()
    
    ec_longitude = dms2dd( deg_long, min_long, sec_long ) # in decimal degrees
    ec_latitude = dms2dd( deg_lat, min_lat, sec_lat ) # in decimal degrees
    
    # Find sine_delta
    sine_delta = ( sin( radians(ec_latitude) ) * cos( radians(obliquity) ) + \
                   cos( radians(ec_latitude) ) * sin( radians(obliquity) ) * \
		   sin( radians(ec_longitude) ) )
    
    delta = asin(sine_delta) # in radians
    delta_deg = degrees(delta) # in decimal degrees
    
    y = ( sin( radians(ec_longitude) ) * cos( radians(obliquity) ) - \
          tan( radians(ec_latitude) ) * sin( radians(obliquity) ) )
    
    x = cos( radians(ec_longitude) )
    
    alpha = degrees( atan(y/x) ) # in degrees
    
    # Removing the ambiguity which arises from taking the inverse tan.
    # The rule is that alpha should lie in the appropriate quadrant:
    # if x > 0 and y > 0 then alpha is between 0->90 degrees
    # if x < 0 and y > 0 then alpha is between 90->180 degrees
    # if x < 0 and y < 0 then alpha is between 180->270 degrees
    # if x > 0 and y < 0 then alpha is between 270->360 degrees
    # alpha must be positive and betweem 0 and 360
    # Add or subtract 180 or 360 to bring it in the correct range.
    #
    #       /
    #   Q2  /  Q1
    #  ///////////
    #   Q3  /  Q4
    #       /
    #
    # Check it in the following order : Q1, Q2, Q3, Q4
    # The if statements define the quadrants and alpha is the angle
    # which must be brought to the correct quadrant
    if ( x > 0 and y > 0 ):
        if alpha > 180 and alpha < 270:
	    alpha = alpha - 180
	elif alpha < -90 and alpha > -180:
	    alpha = alpha + 180
	else:
	    print 'alpha in correct range'
    elif ( x < 0 and y > 0 ):
        if alpha > 270 and alpha < 360:
            alpha = alpha - 180
	elif alpha > -90 and alpha < 0:
            alpha = alpha + 180
	else:
            print 'alpha in correct range'
    elif ( x < 0 and y < 0 ):
        if alpha > 0 and alpha < 90:
            alpha = alpha + 180
	else:
            print 'alpha in correct range'
    elif ( x > 0 and y < 0 ):
        if alpha > 90 and alpha < 180:
            alpha = alpha + 180
	elif alpha < -180 and alpha > -270:
            alpha = alpha + 360 + 180
	elif alpha > -90 and alpha < 0:
	    alpha = alpha + 360
	else:
            print 'alpha in correct range'
    else:
        print 'Unknown range'
    
    # Convert alpha to hours       
    alpha_hours = alpha / 15.0
    
    # Convert from decimals    
    alpha_hms = dh2hms(alpha_hours) # in hms
    delta_dms = dd2dms(delta_deg) # in dms
    
    print 'RA: %.3f, Dec: %.3f' % (alpha_hours, delta_deg)
    return alpha_hours, delta_deg  


####################################################################################
# Commandline run section
if __name__ == '__main__':
    if ( len(argv) != 8 ): 
        print "Error: Incorrect number of arguments. Need 7 input numbers: deg_long, min_long, sec_long, deg_lat, min_lat, sec_lat, obliquity."
	print "Example calling sequence: python ecl2equa.py 139 41 10 4 52 31 23.441884"
	exit()
    
    try:
	ecl2equa( float(argv[1]),  float(argv[2]), float(argv[3]), float(argv[4]),  float(argv[5]), float(argv[6]), float(argv[7]) )
    except ValueError:
        print "Error: Need 7 input numbers: deg_long, min_long, sec_long, deg_lat, min_lat, sec_lat, obliquity."
	print "Example calling sequence: python ecl2equa.py 139 41 10 4 52 31 23.441884"
	exit()
