#!/usr/bin/python
#
# equa2gal
# Program to convert equatorial to galactic coordinates
# input: RA, Dec in decimal hours, degrees
# output: galactic longitude l, galactic latitude b
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Jun 2011
# Modification history: 
# - Y. Tsapras 14 Jun 2011 - version 1.0
########################################################################################

# Import required libraries
from math import cos, sin, tan, asin, acos, atan, atan2
from math import degrees
from math import radians
from dms2dd import dms2dd
from dd2dms import dd2dms
from dh2hms import dh2hms
from hms2dh import hms2dh
from sys import argv, exit

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


########################################################################################
# Function equa2gal starts here
# converts equatorial to galactic coordinates
def equa2gal(hh, mm, ss, degr, mins, secs):
    # check that the input values are all numbers:
    for i in [hh, mm, ss, degr, mins, secs]:
        if ( is_number(i) == False ):
	    print "Error: %s is not a valid number." % str(i)
	    exit()
    
    # convert RA and Dec into decimal form
    declin = dms2dd(degr,mins,secs) # im degrees
    ri_asc = hms2dh(hh,mm,ss) # in hours
    
    # convert right_arc into degrees by multiplying with 15
    ri_asc = ri_asc * 15.0 # in degrees
    
    sin_b = ( cos(radians(declin)) * cos(radians(27.4)) * \
              cos(radians(ri_asc-192.25)) ) + \
	      ( sin(radians(declin)) * sin(radians(27.4)) )
    
    b = degrees(asin(sin_b)) # in degrees
    
    # calculate the nominator for the galactic longitude
    y = sin(radians(declin)) - ( sin(radians(b)) * sin(radians(27.4)) )
    
    # calculate the denominator for the galactic longitude
    x = cos(radians(declin)) * sin(radians(ri_asc-192.25)) * \
             cos(radians(27.4))
    
    # take the inverse tan of the division
    divres = degrees(atan(y/x)) # in degrees
    
    # Removing the ambiguity which arises from taking the inverse tan.
    # The rule is that divres should lie in the appropriate quadrant:
    # if x > 0 and y > 0 then divres is between 0->90 degrees
    # if x < 0 and y > 0 then divres is between 90->180 degrees
    # if x < 0 and y < 0 then divres is between 180->270 degrees
    # if x > 0 and y < 0 then divres is between 270->360 degrees
    # divres must be positive and betweem 0 and 360
    # Add or subtract 180 or 360 to bring it in the correct range.
    #
    #       /
    #   Q2  /  Q1
    #  ///////////
    #   Q3  /  Q4
    #       /
    #
    # Check it in the following order : Q1, Q2, Q3, Q4
    # The if statements define the quadrants and divres is the angle
    # which must be brought to the correct quadrant
    if ( x > 0 and y > 0 ):
        if divres > 180 and divres < 270:
	    divres = divres - 180
	elif divres < -90 and divres > -180:
	    divres = divres + 180
	else:
	    print 'divres in correct range'
    elif ( x < 0 and y > 0 ):
        if divres > 270 and divres < 360:
            divres = divres - 180
	elif divres > -90 and divres < 0:
            divres = divres + 180
	else:
            print 'divres in correct range'
    elif ( x < 0 and y < 0 ):
        if divres > 0 and divres < 90:
            divres = divres + 180
	else:
            print 'divres in correct range'
    elif ( x > 0 and y < 0 ):
        if divres > 90 and divres < 180:
            divres = divres + 180
	elif divres < -180 and divres > -270:
            divres = divres + 360 + 180
	elif divres > -90 and divres < 0:
	    divres = divres + 360
	else:
            print 'divres in correct range'
    else:
        print 'Unknown range'
    
    # add 33 to get the galactic longitude
    l = divres + 33.0 #in degrees
    
    # if the longitude is > 360.0 subtract 360.0
    if (l > 360.0):
        l = l - 360.0
	
    # Convert from decimals into minutes and seconds   
    l_dms = dd2dms(l) # in dms
    b_dms = dd2dms(b) # in dms
    
    #print 'Galactic Longitude l: %.3f, Galactic Latitude b: %.3f degrees' % (l, b)
    # return the decimal forms
    return l, b  


####################################################################################
# Commandline run section
if __name__ == '__main__':
    if ( len(argv) != 7 ): 
        print "Error: Incorrect number of arguments. Need 6 input numbers: RA(hr,min,sec) and Dec(deg,arcmin,arcsec)."
	print "Example calling sequence: python equa2gal.py 10 21 00 10 03 11"
	exit()
    
    try:
	equa2gal( float(argv[1]),  float(argv[2]), float(argv[3]), float(argv[4]),  float(argv[5]), float(argv[6]) )
    except ValueError:
        print "Error: Need 6 input numbers: RA(hr,min,sec) and Dec(deg,arcmin,arcsec)."
	print "Example calling sequence: python equa2gal.py 10 21 00 10 03 11"
	exit()
