#!/usr/bin/python
#
# gal2equa
# Program to convert galactic to equatorialcoordinates
# input: galactic longitude l, galactic latitude b (deg,min,sec)
# output: RA, Dec in decimal hours, degrees
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Jun 2011
# Modification history: 
# - Y. Tsapras 14 Jun 2011 - version 1.0
########################################################################################

# Import required libraries
from math import cos, sin, tan, asin, acos, atan, atan2
from math import degrees
from math import radians
from dms2dd import dms2dd
from dd2dms import dd2dms
from dh2hms import dh2hms
from hms2dh import hms2dh
from sys import argv, exit

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


########################################################################################
# Function equa2gal starts here
# converts equatorial to galactic coordinates
def gal2equa(ld, lm, ls, bd, bm, bs):
    # check that the input values are all numbers:
    for i in [ld, lm, ls, bd, bm, bs]:
        if ( is_number(i) == False ):
	    print "Error: %s is not a valid number." % str(i)
	    exit()
    
    # convert l and b into decimal form
    l = dms2dd(ld, lm, ls) # im degrees
    b = dms2dd(bd, bm, bs) # in degrees
    
    # find sin(declin)
    sin_d = ( cos(radians(b)) * cos(radians(27.4)) * \
              sin(radians(l-33.0)) ) + \
	      ( sin(radians(b)) * sin(radians(27.4)) )
    
    # take inverse sin to find the declination d in degrees
    declin = degrees(asin(sin_d)) # in degrees
    
    # calculate the nominator for the right ascension
    y = cos(radians(b)) * cos(radians(l-33.0))
    
    # calculate the denominator for the right ascension
    x = sin(radians(b)) * cos(radians(27.4)) - \
            cos(radians(b)) * sin(radians(27.4)) * sin(radians(l-33.0))
    
    # take the inverse tan of the division
    divres = degrees(atan(y/x)) # in degrees
    
    # Removing the ambiguity which arises from taking the inverse tan.
    # The rule is that divres should lie in the appropriate quadrant:
    # if x > 0 and y > 0 then divres is between 0->90 degrees
    # if x < 0 and y > 0 then divres is between 90->180 degrees
    # if x < 0 and y < 0 then divres is between 180->270 degrees
    # if x > 0 and y < 0 then divres is between 270->360 degrees
    # divres must be positive and betweem 0 and 360
    # Add or subtract 180 or 360 to bring it in the correct range.
    #
    #       /
    #   Q2  /  Q1
    #  ///////////
    #   Q3  /  Q4
    #       /
    #
    # Check it in the following order : Q1, Q2, Q3, Q4
    # The if statements define the quadrants and divres is the angle
    # which must be brought to the correct quadrant
    if ( x > 0 and y > 0 ):
        if divres > 180 and divres < 270:
    	   divres = divres - 180
        elif divres < -90 and divres > -180:
    	   divres = divres + 180
        else:
    	   print 'divres in correct range'
    elif ( x < 0 and y > 0 ):
    	if divres > 270 and divres < 360:
    	    divres = divres - 180
        elif divres > -90 and divres < 0:
    	    divres = divres + 180
        else:
    	    print 'divres in correct range'
    elif ( x < 0 and y < 0 ):
    	if divres > 0 and divres < 90:
    	    divres = divres + 180
        else:
    	    print 'divres in correct range'
    elif ( x > 0 and y < 0 ):
    	if divres > 90 and divres < 180:
    	    divres = divres + 180
        elif divres < -180 and divres > -270:
    	    divres = divres + 360 + 180
        elif divres > -90 and divres < 0:
    	   divres = divres + 360
        else:
    	    print 'divres in correct range'
    else:
    	print 'Unknown range'
    
    # add 192.25 to get the right ascension
    ra = divres + 192.25 #in degrees
    
    # divide by 15 to get the right ascension in hours
    ra = ra/15.0 # in hours
    
    # if the hours are > 24, subtract 24
    if (ra > 24.0):
        ra = ra -24.0
	
    # Convert from decimals into minutes and seconds   
    ra_dms = dh2hms(ra) # in hms
    dec_dms = dd2dms(declin) # in dms
    
    print 'RA: %.3f, Dec: %.3f degrees' % (ra, declin)
    # return the decimal forms
    return ra, declin  


####################################################################################
# Commandline run section
if __name__ == '__main__':
    if ( len(argv) != 7 ): 
        print "Error: Incorrect number of arguments. Need 6 input numbers: galactic long(deg,min,sec), galactic lat(deg,min,sec)."
	print "Example calling sequence: python gal2equa.py 232 14 52 51 07 20"
	exit()
    
    try:
	gal2equa( float(argv[1]),  float(argv[2]), float(argv[3]), float(argv[4]),  float(argv[5]), float(argv[6]) )
    except ValueError:
        print "Error: Need 6 input numbers: Need 6 input numbers: galactic long(deg,min,sec), galactic lat(deg,min,sec)."
	print "Example calling sequence: python gal2equa.py 232 14 52 51 07 20"
	exit()
