#!/usr/bin/python
#
# gst2ut
# Program to convert Greenwich mean sidereal time to universal time
# This calculates the value for the beginning of the day (UT 0h) 
# input: GST (in hr, min, sec)
# output: hrs,min,sec,dd,mm,yyyy
#
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Feb 2009
#
# Modification history
#  YT - 4 May 2011 - Added catches for input errors. 
#  YT - 4 May 2011 - Added command line call.
####################################################################################

# Import required modules
from julday import julday
from hms2dh import hms2dh
from dh2hms import dh2hms
from sys import argv, exit

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


####################################################################################
# Function gst2ut starts here
# converts Greenwich mean sidereal time to universal time
def gst2ut(hrs, mins, sec, dd, mm, yyyy):
    # Catch exception if input is not a number
    try:
        float(hrs)
	if ( float(hrs) < 0 or float(hrs) > 24):
	    print "Error: gst2ut : First input element must be a valid hour number between 0 and 24."
	    exit()
    except ValueError:
        print "Error: gst2ut : First input element must be a valid hour number between 0 and 24."
	exit()
    # Check that the mins, sec input values are sensible
    for i in (mins, sec):
        if ( is_number(i) == False ):
	    print "Error: gst2ut : %s is not a valid number." % str(i)
	    exit()
	else:
	    if ( i < 0 or i  > 60 ):
	        print "Error: gst2ut : Input minutes and seconds must have values between 0 and 60."
	        exit()
    # Convert calendar date to julian date
    if ( is_number(dd) == False or is_number(mm) == False or is_number(yyyy) == False ):
        print "Error: gst2ut : dd, mm, yyyy must be numbers."
	exit()    
    jd = julday(dd, mm, yyyy)
    S = jd - 2451545.0
    T = S / 36525.0
    T0 = 6.697374558 + (2400.051336 * T) + (0.000025862 * T * T)

    # Reduce the result to the range 0-24 by adding or subtracting multiples of 24
    checking = True
    while checking:
        if (T0 < 0):
            T0 = T0 + 24
        elif (T0 > 24):
            T0 = T0 - 24
        else:
            checking = False #This causes the while loop to stop

    # Convert GST to decimal hours
    gstdh = hms2dh(hrs, mins, sec)
    T0 = gstdh - T0

    # Reduce the result to the range 0-24 by adding or subtracting multiples of 24
    # The result is the GST
    checking = True
    while checking:
        if (T0 < 0):
            T0 = T0 + 24
        elif (T0 > 24):
            T0 = T0 - 24
        else:
            checking = False #This causes the while loop to stop
    UT = T0 * 0.9972695663

    # Convert the result to hrs, min, sec
    UT = dh2hms(UT)
    print 'Universal time: %d:%d:%.2f' % (UT[0],UT[1],UT[2])
    return UT


####################################################################################
# Commandline run section
if __name__ == '__main__':
    if ( len(argv) != 7 ): 
        print "Error: Incorrect number of arguments. Input must have 6 elements: hrs, mins, sec, dd, mm, yyyy"
	print 'Example calling sequence: python gst2ut.py 4 40 5.23 22 4 1980'
	exit()
    
    try:
        gst2ut( float(argv[1]),  float(argv[2]), float(argv[3]), float(argv[4]), float(argv[5]), float(argv[6]) )
	# print 'Local sidereal time:', LST
    except ValueError:
        print "Error: Incorrect number of arguments. Input must have 6 elements: hrs, mins, sec, dd, mm, yyyy"
	print "Example calling sequence: python gst2ut.py 4 40 5.23 22 4 1980"
	exit()
