#!/usr/local/bin/python
#
# julday
# Program to convert a given calendar date to a Julian date
# input: day,month,year (also accepts decimal days)
# output: JD 
#
# get_julian
# Program to convert a given calendar date to a Julian date
# input: list of 3 or 6 numbers: [dd, mm, yyyy, *hrs, *min, *sec]
# output: JD
#  --> * signifies optional input, if they are not provided JD is returned for 0h 0m 0s
#
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Feb 2009
#
# Modification history
#
# Y. Tsapras 4 May 2011 - Added error catch
# Y. Tsapras 16 May 2011 - Added support for hrs, min, sec
####################################################################################

# Import required libraries
from sys import argv, exit
from hms2dh import hms2dh

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False



####################################################################################
# Sanity check procedure to ensure valid input
def do_check( hh, mm, ss ):
    for val in [ hh, mm, ss ]:
 	if (type(val).__name__=='int' or  type(val).__name__=='float'):
 	    check_id = '0'
 	    continue
 	else:
 	    check_id = '1'
	    print "Data type is invalid. %s is not an integer or floating point number." % str(val)
	    exit()
    return check_id


####################################################################################
# Convert calendar date to julian date
# Accepts decimal days
# If a decimal date has not been given, return JD for 0h0m0s
def julday( dd, mm, yyyy ):
    # run sanity check
    try:
        do_check( float(dd), int(mm), int(yyyy) )
    except ValueError:
        print "Error: julday requires 3 numbers as input: dd, mm, yyyy"
	exit()
    if ( mm == 1 ) or ( mm == 2 ):
        yyyy = yyyy - 1
        mm = mm + 12
 
    A = int( yyyy / 100 )
    B = 2 - A + int(A/4)

    if ( yyyy < 0 ):
        C = int( (365.25 * yyyy) - 0.75 )
 
    else:
        C = int( 365.25 * yyyy )
        D = int( 30.6001 * (mm + 1) )

    JD = B + C + D + dd + 1720994.5
    print 'JD=', JD
    return JD


###############################################################################
# Convert input to a valid julian date
# Input is a list of 3 or 6 numbers: dd, mm, yyyy [, hrs, min, sec]
def get_julian( in_arr ):
    # If only a julian date has been given...
    if ( len(in_arr) == 1 ):
        try:
            julian_date = float(in_arr[0])
	except ValueError:
	    print "Error: get_julian -> input JD must be a valid floating point number."
	    print "Check your input value."
	    exit()
    # If only a date has been given without hour, min, sec information assume 0h0m0s
    elif ( len(in_arr) == 3 ):
        try:
            julian_date = julday( float(in_arr[0]), int(in_arr[1]), int(in_arr[2]) )
	except ValueError:
	    print "Error: get_julian -> accepts a list with 3 numbers as input: [dd, mm, yyyy]."
	    print "Check your input values."
            exit()
    # If a date has been given with full day, month, year, hour, min, sec information...
    elif ( len(in_arr) == 6 ):
        day, month, year, hrs, mins, sec = in_arr[0], in_arr[1], in_arr[2], in_arr[3], in_arr[4], in_arr[5]
        # Catch exception if input is not a number
        try:
            float(hrs)
	    if ( float(hrs) < 0 or float(hrs) > 24):
	        print "Error: input value for the hour must be a number between 0 and 24."
	        exit()
        except ValueError:
            print "Error: input value for the hour must be a number between 0 and 24."
	    exit()
        # Check that the mins, sec input values are sensible
        for i in (mins, sec):
            if ( is_number(i) == False ):
	        print "Error: %s is not a valid number." % str(i)
	        exit()
	    else:
	        if ( float(i) < 0 or float(i)  > 60 ):
	            print "Error: Input minutes and seconds must have values between 0 and 60."
	            exit()
        # Check that the day, month and year values are valid
	for j in (day, month, year):
	    if ( is_number(j) == False ):
                print "Error: %s is not a valid number." % str(j)
                exit()
	# Convert the hour, min, sec information to decimal hours and add them to the day
	add_to_day = hms2dh( float(hrs), float(mins), float(sec) )
        day_plus = float(day) + ( add_to_day/24. )
	julian_date = julday( day_plus, float(month), float(year) )
    else:
        print "Error: Incorrect number of arguments passed to get_julian."
	print "get_julian accepts a list with 1, 3 or 6 elements."
	print '''Input : julian date (1 argument) OR 
	         calendar date (6 arguments: day, month, year, hrs, min, sec) OR
                 calendar date (3 arguments: day, month, year - assumes 0h 0m 0s)'''
        print "e.g. get_julian( [2449454.5] )"
	exit()
    return julian_date
    


####################################################################################
# Allow calls from the prompt
# e.g python julday.py 21 5 2010
if __name__ == "__main__":
    if ( len(argv) != 4 and len(argv) != 7 ):
        print "Error: Incorrect number of arguments. julday.py requires 3 or 6 numbers: dd, mm, yyyy [, hrs, min, sec]"
	print "e.g. python julday.py 16 4 2011"
	exit() 
    # run sanity check
    try:
        JD = get_julian( argv[1:] )
    except ValueError:
        print "Error: Incorrect number of arguments. julday.py requires 3 or 6 arguments:"
	print '''Input: python julday.py <space separated list of arguments>
	         where the arguments are:
	         calendar date (6 arguments: day, month, year, hrs, min, sec) OR
	         calendar date (3 arguments: day, month, year - assumes 0h 0m 0s)'''
	print 'e.g. python julday.py 16 4 2011'
	exit() 
