#!/usr/local/bin/python
#
# moon_pos
# Calculates the position of the Moon for a given date
# Input is a list containing 1, 3 or 6 numbers.
# Input : julian date (1 argument) OR calendar date (6 arguments: day, month, year, hrs, min, sec) OR
#         calendar date (3 arguments: day, month, year - assumes 0h 0m 0s)
# Output: moon_pos (RA, Dec, Distance) - in decimal hours, decimal degrees and
#  kilometers respectively
#
# The accuracy of the results will be approximately 10 arcsec in the longitude and 4 arcsec in the latitude 
# for the position of the Moon.
#
# Equations based on Astronomical Algorithms by Jean Meeus (Chapter 45, p307)
#
# Written by Y.Tsapras Feb 2009
#
# Modification history
# Y. Tsapras 4 May 2011 - modified to work for different input arguments, including JD
# Y. Tsapras 16 May 2011 - fixed an error with the calculation of the F value
###############################################################################

# Import required libraries
from math import cos, sin, acos
from math import degrees
from math import radians
from rd_tables import rd_tables
from julday import julday
from dms2dd import dms2dd
from obliq_eclip import obliq_eclip
from ecl2equa import ecl2equa
from dd2dms import dd2dms
from dh2hms import dh2hms
from nutation import nutation
from hms2dh import hms2dh
from sys import argv, exit

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


###############################################################################
# Function reduce_angle
# To avoid working with large angles, reduce them to less
# than 360 degrees
def reduce_angle( big_angle ):
    while big_angle > 360:
        big_angle = big_angle - 360
    
    while big_angle < 0:
        big_angle = big_angle + 360
    
    out_angle = big_angle
    return out_angle


###############################################################################
# Convert input to a valid julian date
def get_julian( in_arr ):
    # If only a julian date has been given...
    if ( len(in_arr) == 1 ):
        try:
            julian_date = float(in_arr[0])
	except ValueError:
	    print "Error: get_julian -> input JD must be a valid floating point number."
	    print "Check your input value."
	    exit()
    # If only a date has been given without hour, min, sec information assume 0h0m0s
    elif ( len(in_arr) == 3 ):
        try:
            julian_date = julday( float(in_arr[0]), int(in_arr[1]), int(in_arr[2]) )
	except ValueError:
	    print "Error: get_julian -> accepts a list with 3 numbers as input: [dd, mm, yyyy]."
	    print "Check your input values."
            exit()
    # If a date has been given with full day, month, year, hour, min, sec information...
    elif ( len(in_arr) == 6 ):
        day, month, year, hrs, mins, sec = in_arr[0], in_arr[1], in_arr[2], in_arr[3], in_arr[4], in_arr[5]
        # Catch exception if input is not a number
        try:
            float(hrs)
	    if ( float(hrs) < 0 or float(hrs) > 24):
	        print "Error: input value for the hour must be a number between 0 and 24."
	        exit()
        except ValueError:
            print "Error: input value for the hour must be a number between 0 and 24."
	    exit()
        # Check that the mins, sec input values are sensible
        for i in (mins, sec):
            if ( is_number(i) == False ):
	        print "Error: %s is not a valid number." % str(i)
	        exit()
	    else:
	        if ( float(i) < 0 or float(i)  > 60 ):
	            print "Error: Input minutes and seconds must have values between 0 and 60."
	            exit()
        # Check that the day, month and year values are valid
	for j in (day, month, year):
	    if ( is_number(j) == False ):
                print "Error: %s is not a valid number." % str(j)
                exit()
	# Convert the hour, min, sec information to decimal hours and add them to the day
	add_to_day = hms2dh( float(hrs), float(mins), float(sec) )
        day_plus = float(day) + ( add_to_day/24. )
	julian_date = julday( day_plus, float(month), float(year) )
    else:
        print "Error: Incorrect number of arguments passed to get_julian."
	print "get_julian accepts a list with 1, 3 or 6 elements."
	print '''Input : julian date (1 argument) OR 
	         calendar date (6 arguments: day, month, year, hrs, min, sec) OR
                 calendar date (3 arguments: day, month, year - assumes 0h 0m 0s)'''
        print "e.g. get_julian( [2449454.5] )"
	exit()
    return julian_date

###############################################################################
# Function moon_pos starts here
# Calculates the position of the Moon for a given date
def moon_pos( julian_date ):
    [table1,size_table1] = rd_tables("table1.txt")
    [table2,size_table2] = rd_tables("table2.txt")
    
    JD = julian_date
    T = (JD - 2451545.0) / 36525.0
    T2 = T * T
    T3 = T2 * T
    T4 = T3 * T

    # Calculate the angles L1, D, M, M1, F

    # The Moon's mean longitude (and including the constant 
    #  term of the effect of light-time):
    L1 = ( 218.3164591 + (481267.88134236 * T) - \
          (0.0013268 * T2) + (T3 / 538841.0) - (T4 \
          / 65194000.0) )

    L1 = reduce_angle( L1 ) # in degrees
              
    # Mean elongation of the Moon:
    D = ( 297.8502042 + (445267.1115168 * T) - \
         (0.0016300 * T2) + (T3 / 545868.0) - (T4 \
         / 113065000.0) )

    D = reduce_angle( D ) # in degrees
         
    # Sun's mean anomaly:
    M = ( 357.5291092 + (35999.0502909 * T) - \
         (0.0001536 * T2) + (T3 / 24490000.0) )

    M = reduce_angle( M ) # in degrees
    
    # Moon's mean anomaly:
    M1 = ( 134.9634114 + (477198.8676313 * T) + \
          (0.0089970 * T2) + (T3 / 69699.0) - \
          (T4 / 14712000.0) )

    M1 = reduce_angle( M1 ) # in degrees
    
    # Moon's argument of latitude (mean distance of the 
    #  Moon from it's ascending node):
    F = ( 93.2720993 + (483202.0175273 * T) - \
         (0.0034029 * T2) - (T3 / 3526000.0) + \
         (T4 / 863310000.0) )

    F = reduce_angle( F ) # in degrees
    
    # Three further arguments in degrees are needed:
    A1 = reduce_angle( 119.75 + ( 131.849 * T ) ) # in degrees
    A2 = reduce_angle( 53.09 + ( 479264.290 * T ) ) # in degrees
    A3 = reduce_angle( 313.45 + ( 481266.484 * T ) ) # in degrees
    
    # Calculate correction needed due to the eccentricity
    #  of the Earth's orbit around the Sun:
    E = 1.0 - (0.002516 * T) - (0.0000074 * T2)
    E2 = E * E
        
    # Calculate the sums sig1, sigr and sigb of the tables
    # The argument of each sine and cosine is a linear combination
    # of the four fundamental arguments D, M, M1 and F (the first four
    # terms in the tables). For example, the argument on the 8th line
    # of table1 is 2*D -1*M -1*M1 and the contributions to sig1 and
    # sigr are +57066 sin(2*D -1*M -1*M1) and -152138 cos(2*D -1*M -1*M1)
    # respectively.

    sig1 = 0.0
    sigr = 0.0
    sigb = 0.0
    
    # The terms whose argument contains the angle M depend on the
    # eccentricity of the Earth's orbit around the Sun. The amplitude
    # of these terms is variable. To take this effect into account,
    # we must multiply the terms whose argument contains M or -M by E
    # and those containing 2M or -2M with E**2 (i.e. E2) 
    for elem in table1:
        # Obtain the M coefficient for this line
        Mcoeff =  int(elem[1])
	# Replace the M coefficient if it is 1 or -1
	if (Mcoeff == 1 or Mcoeff == -1):
	    Mcoeffok = Mcoeff * E
	# Replace the M coefficient if it is 2 or -2
	elif (Mcoeff == 2 or Mcoeff == -2):
	    Mcoeffok = Mcoeff * E2
	# Otherwise leave it alone
	else:
	    Mcoeffok = Mcoeff
	#DEBUGGING STATEMENT: print "M = "+elem[1]+" and Mcoeff is "+str(Mcoeffok)
        term1 = ( ( int(elem[0]) * D ) + ( Mcoeffok * M ) + \
                  ( int(elem[2]) * M1 ) + ( int(elem[3]) * F) )
        sig1 = sig1 + ( int(elem[4]) * sin(radians(term1)) )
        sigr = sigr + ( int(elem[5]) * cos(radians(term1)) )

    for ele in table2:
        Mcoeff2 =  int(ele[1])
        if (Mcoeff2 == 1 or Mcoeff2 == -1):
	    Mcoeff2ok = Mcoeff2 * E
        elif (Mcoeff2 == 2 or Mcoeff2 == -2):
	    Mcoeff2ok = Mcoeff2 * E2
        else:
	    Mcoeff2ok = Mcoeff2
	#DEBUGGING STATEMENT:print "M = "+ele[1]+" and Mcoeff2 is "+str(Mcoeff2ok)	  
        term2 = ( ( int(ele[0]) * D ) + ( Mcoeff2ok * M ) + \
                  ( int(ele[2]) * M1 ) + ( int(ele[3]) * F ) )
        sigb = sigb + ( int(ele[4]) * sin(radians(term2)) )
          
    # Calculate extra additive terms due to the actions of
    #  Venus and Jupiter:
    # Additive to sig1:
    add_sig1 = ( ( 3958.0 * sin(radians(A1)) ) + ( 1962.0 * sin(radians(L1 - F)) ) + \
                 ( 318.0 * sin(radians(A2)) ) )

    # Additive to sigb:
    add_sigb = ( ( -2235.0 * sin(radians(L1)) ) + ( 382.0 * sin(radians(A3)) ) + \
                 ( 175.0 * sin(radians(A1 - F)) ) + ( 175.0 * sin(radians(A1 + F)) ) + \
	         ( 127.0 * sin(radians(L1 - M1)) ) - ( 115.0 * sin(radians(L1 + M1)) ) )

    sig1 = sig1 + add_sig1
    sigb = sigb + add_sigb

    # The coordinates for the Moon are:
    lamda = L1 + ( sig1 / 1000000.0 ) # in decimal degrees
    
    beta = sigb / 1000000.0 # in decimal degrees
    
    delta = 385000.56 +  ( sigr / 1000.0 )# in kilometers

    # The apparent longitude of the Moon is obtained by adding to lamda
    #  the nutation in longitude (delta_psi)
    [nutation_long, nutation_lat] = nutation( JD ) # in arcsec
    nutation_long = nutation_long / 3600. # in degrees
    nutation_lat = nutation_lat / 3600. # in degrees
    
    # Add nutation in longitude to lamda
    lamda = lamda + nutation_long
    
    # Calculate obliquity
    obliquity = obliq_eclip( JD ) # in decimal degrees

    # Add nutation in latitude to obliquity
    # obliquity = obliquity + nutation_lat     

    # Convert to dms
    lamda_dms = dd2dms(lamda) # in dms
    beta_dms = dd2dms(beta) #in dms

    # Calculate RA, Dec
    [right_asc,declin] = ecl2equa( lamda_dms[0],lamda_dms[1],lamda_dms[2], \
                                   beta_dms[0],beta_dms[1],beta_dms[2], \
				   obliquity )
    # Bring RA to range 0-24
    while right_asc > 24:
        right_asc = right_asc - 12

    while right_asc < 0:
        right_asc = right_asc + 12

    print '--Moon-- RA: %.3f, Dec: %.3f' % (right_asc, declin)
    print 'Moon Distance: %.3f km' % delta
    return right_asc, declin, delta


############################################################################### 
# Allow calls from the prompt
# e.g python moon_pos.py 2449454.5
if __name__ == "__main__":
    if ( len(argv) != 2 and len(argv) != 4 and len(argv) != 7):
        print "Error: Incorrect number of arguments. moon_pos.py accepts 1,3 or 6 arguments:"
	print '''Input: python moon_pos.py Julian Date (1 argument) OR 
	         calendar date (6 arguments: day, month, year, hrs, min, sec) OR
	         calendar date (3 arguments: day, month, year - assumes 0h 0m 0s)'''
	print 'e.g. python moon_pos.py 2449454.5'
	exit() 
    # run sanity check
    try:
        JD = get_julian( argv[1:] )
	moon_pos( JD )
    except ValueError:
        print "Error: Incorrect number of arguments. moon_pos.py accepts 1,3 or 6 arguments:"
	print '''Input: python moon_pos.py Julian Date (1 argument) OR 
	         calendar date (6 arguments: day, month, year, hrs, min, sec) OR
	         calendar date (3 arguments: day, month, year - assumes 0h 0m 0s)'''
	print 'e.g. python moon_pos.py 2449454.5'
	exit() 
