#!/usr/local/bin/python
#
# nutation
# Calculates the nutation
# The nutation is a periodic oscillation of the rotational axis of the Earth
#  around it's 'mean' position.
# Input : Julian Date (J2000)
# Output: nutation in longitude, nutation in obliquity (in arcsec)
#
# Equations based on Astronomical Algorithms by Jean Meeus
#
# Approximations accurate to 0.5 arcsec in delta_psi and
#  0.1 arcsec in delta_epsilon
#
# Written by Y.Tsapras Feb 2009
#
# Modification history
# Y.Tsapras Apr 2010 - updated imports
# Y.Tsapras 11 May 2011 - changed input from dd,mm,yyyy to JD
# Y.Tsapras 12 May 2011 - updated equations for higher accuracy
###############################################################################

# Import required libraries
from math import cos, sin, acos
from math import degrees
from math import radians
from rd_tables import rd_tables
from dms2dd import dms2dd
from ecl2equa import ecl2equa
from dd2dms import dd2dms
from dh2hms import dh2hms
from rd_tables import rd_tables

###############################################################################
# Function reduce_angle
# To avoid working with large angles, reduce them to less
# than 360 degrees
def reduce_angle( big_angle ):
    while big_angle > 360:
        big_angle = big_angle - 360
    
    while big_angle < 0:
        big_angle = big_angle + 360
    
    out_angle = big_angle
    return out_angle

###############################################################################
# Function nutation starts here
# Calculates the nutation for a given JD
def nutation( JD ):
    T = (JD - 2451545.0) / 36525.0
    T2 = T * T
    T3 = T2 * T
    T4 = T3 * T
    
    # Read in the tables of the coefficients
    [table3,size_table3] = rd_tables("table3.txt")
    
    # Mean elongation of the Moon from the Sun:
    D = ( 297.85036 + (445267.111480 * T) - (0.0019142 * T2) + \
          (T3 / 189474.0) )
    
    D = reduce_angle(D)
    
    # Mean anomaly of the Sun (Earth):
    M = ( 357.52772 + (35999.050340 * T) - \
         (0.0001603 * T2) - (T3 / 300000.0) )
    
    M = reduce_angle( M )
    
    # Mean anomaly of the Moon:
    M1 = ( 134.96298 + (477198.867398 * T) + \
          (0.0086972 * T2) + (T3 / 56250.0) )
    
    M1 = reduce_angle( M1 )
    
    # Moon's argument of latitude (mean distance of the 
    #  Moon from it's ascending node):
    F = ( 93.27191 + (483202.017538 * T) - \
         (0.0036825 * T2) + (T3 / 327270.0) )
    
    F = reduce_angle ( F )
    
    # Longitude of the ascending node of the Moon's mean orbit on
    #  the ecliptic measured from the mean equinox of the date
    omega = ( 125.04452 - (1934.136261 * T) + (0.0020708 * T2) + \
              (T3 / 450000.0) )
    
    omega = reduce_angle (omega )
    
    # Calculate the sums sum_delta_psi and sum_delta_epsilon from the table
    # The argument of each sine (for delta_psi)and cosine (for delta_epsilon) 
    # is a linear combination of the five fundamental arguments D, M, M1, F and omega 
    # (the first five terms in the table). For example, the argument on the 2nd line
    # of table1 is -2*D + 2*F + 2*omega and the contributions to sum_delta_psi and
    # sum_delta_epsilon are (-13187 -1.6*T) * sin(-2*D + 2*F + 2*omega) and 
    # (5736 -3.1*T) * cos(-2*D + 2*F + 2*omega)
    # respectively.
    
    sum_delta_psi = 0
    sum_delta_epsilon = 0
    
    # Go through the rows of the tables
    for elem in table3:
        # Argument
        term1 = ( (int(elem[0]) * D) + (int(elem[1]) * M) + \
                  (int(elem[2]) * M1) + (int(elem[3]) * F) + \
		  (int(elem[4]) * omega) )
	
        # Coefficient of the sine of the Argument
	coeff_sine_term1 = int(elem[5]) + (float(elem[6]) * T)
	
        # Coefficient of the cosine of the Argument
	coeff_cosine_term1 = int(elem[7]) + (float(elem[8]) * T)
	
        sum_delta_psi = sum_delta_psi + ( coeff_sine_term1 * sin( radians(term1) ) )
        sum_delta_epsilon = sum_delta_epsilon + ( coeff_cosine_term1 * cos( radians(term1) ) )
    
    # Get the values for the nutation in longitude, delta_psi, and
    # the nutation in obliquity, delta_epsilon, by multiplying by the units used in the table (0.0001 arcsec) 
    delta_psi = sum_delta_psi * 0.0001 # in arcseconds
    delta_epsilon = sum_delta_epsilon * 0.0001 # in arcseconds   
    
    print 'Nutation in longitude: %.4f arcseconds' % delta_psi
    print 'Nutation in obliquity: %.4f arcseconds' % delta_epsilon
    
    return delta_psi, delta_epsilon
