#!/usr/bin/python
#
# obliq_eclip
# Program to calculate the value of the obliquity of the ecliptic
# input: Julian Date (J2000)
# output: obliquity (in decimal degrees)
# "obliquity" is the obliquity of the ecliptic, or the inclination of the Earth's axis of rotation. It
# is the angle between the planes of the equator and the ecliptic. 
#
# Equations based on Astronomical Algorithms by Jean Meeus
#
# Written by Y.Tsapras Feb 2009
#
# Modification history
# Y.Tsapras Apr 2010 - updated imports
# Y.Tsapras 11 May 2011 - changed input from dd,mm,yyyy to JD
# Y.Tsapras 12 May 2011 - updated equations for higher accuracy (using the expression of Laskar 1986, A&A 157, 68)
###############################################################################


# Import required libraries
from nutation import nutation


###############################################################################
# Function obliq_eclip starts here
# calculates the value of the obliquity of the ecliptic
def obliq_eclip( JD ):
    # T is the time measured in Julian centuries from the epoch J2000.0
    T = (JD - 2451545.0) / 36525.0
    
    # U is the time measured in units of 10000 Julian years from J2000.0
    U = []
    U0 = T/100.
    # Work out the powers of U up to the 10th power and append them to U
    for i in range(0,10): U.append( U0**(i+1) )
    
    # The mean obliquity is epsilon_0
    # 23.43929111 * 60.0 * 60.0 = 84381.447996000003 -> in arcseconds
    epsilon_0 = ( 84381.447996000003 - (4680.93 * U[0]) \
                  - (1.55 * U[1]) + (1999.25 * U[2]) \
	          - (51.38 * U[3]) - (249.67 * U[4]) \
                  - (39.05 * U[5]) + (7.12 * U[6]) \
                  + (27.87 * U[7]) + (5.79 * U[8]) \
                  + (2.45 * U[9]) ) # in arcseconds
    
    # The true obliquity ofthe ecliptic is epsilon = epsilon_0 + deltra_epsilon, where
    # delta_epsilon is the nutation in obliquity
    delta_psi, delta_epsilon = nutation( JD )   # in arcseconds
      
    obliquity = (epsilon_0 + delta_epsilon)/3600. # in decimal degrees
    print 'Mean obliquity = %.4f degrees' % (epsilon_0/3600.)
    print 'True obliquity = %.4f degrees' % obliquity
    return obliquity
