#!/usr/bin/python
#
# rise_set
# Program to calculate the rise and set times of an object in the sky.
# It returns the times at which the altitude reaches 0 degrees.
# It does not account for parallax.
#
# input: ([coordinates, latitude, longitude, date]) : ([RA, Dec, latitude, longitude, date]) 
#  : ([hrs, min, sec, deg, arcmin, arcsec, deg, direction, deg, direction, dd, mm, yyyy])
#   where latitude is a 2 element array and direction is 0,"N" (North) or 1,"S" (South)
#   and longitude is a 2 element array and direction is 0 ,"W"(West) or 1,"E" (East)
#   and date is in dd, mm, yyyy 
#  Total of 13 parameters
# e.g coords_1=[17, 32, 11, -18, 12, 03, 19.825, 0, -155.285, 0, 6, 4, 2011]
# e.g.2 coords_1=[23, 39, 20, 21, 42, 00, 30, 0, 64, 1, 24, 8, 1980]
# output: rise and set times ([rise_hr, rise_min, rise_sec, set_hr, set_min, set_sec])
#
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
#
# Written by Y.Tsapras Feb 2009
#
# Modification history:
# Y. Tsapras 18 May 2010 - version 1.0
# Y. Tsapras 16 May 2011 - added error reporting and command line call
# Y. Tsapras 16 May 2011 - added support for string input latitude and 
#                          longitude directions ("N","S","E","W") are now valid
####################################################################################

from math import cos, tan, sin, acos, pi, asin
from math import degrees, radians
from hms2dh import hms2dh
from dh2hms import dh2hms
from dms2dd import dms2dd
from dd2dms import dd2dms
from gst2ut import gst2ut
from lst2gst import lst2gst
from sys import exit, argv

####################################################################################
# Check if a string is a number
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


####################################################################################
# Function rise_set starts here
def rise_set(coords_1):
    # Check the the input coordinates are valid
    if (len(coords_1) != 13):
         print "Error: Incorrect number of arguments. rise_set requires a list of 13 input arguments:" 
	 print " corresponding to: RA(target), Dec(target), latitude(observatory), longitude(observatory), date"
	 print "e.g. [23, 39, 20, 21, 42, 00, 30, 0, 64, 1, 24, 8, 1980]"
	 print "Aborting!"
	 exit()
    
    # Check that RA and Dec are valid numbers
    for i in coords_1[0:6]:
        if (is_number(i) == False):
	    print "Error: rise_set requires the RA and Dec to be valid numbers. %s is not a valid number" % str(i)
            print "rise_set requires a list of 13 input arguments:" 
	    print " corresponding to: RA(target), Dec(target), latitude(observatory), longitude(observatory), date"
	    print "e.g. [23, 39, 20, 21, 42, 00, 30, 0, 64, 1, 24, 8, 1980]"
	    print "Aborting!"
	    exit()
    	
    # Check that the latitude is a valid input
    if ( is_number(coords_1[6]) == False ):
        print "Error: rise_set : Latitude must be a number."
        exit()
    # Check that the latitude input direction is either 0, 1, "N" or "S"
    direction_lat = coords_1[7]
    if ( direction_lat not in [0, 1, "N", "S", "0", "1"] ):
        print 'Error: rise_set : Invalid input for direction_lat: valid options are 0, 1, "N", "S" '
        exit()
    else:
        if ( direction_lat == "N" ): coords_1[7] = 0
        if ( direction_lat == "S" ): coords_1[7] = 1
        if ( direction_lat == "0" ): coords_1[7] = 0
        if ( direction_lat == "1" ): coords_1[7] = 1
    
    # Check that the longitude is a valid input
    if ( is_number(coords_1[8]) == False ):
        print "Error: rise_set : Longitude must be a number."
        exit()
    # Check that the longitude input direction is either 0, 1, "W" or "E"
    direction_lon = coords_1[9]
    if ( direction_lon not in [0, 1, "W", "E", "0", "1"] ):
        print 'Error: rise_set : Invalid input for direction_lon: valid options are 0, 1, "W", "E" '
        exit()
    else:
        if ( direction_lon == "W" ): coords_1[9] = 0
        if ( direction_lon == "E" ): coords_1[9] = 1
        if ( direction_lon == "0" ): coords_1[9] = 0
        if ( direction_lon == "1" ): coords_1[9] = 1
    
    # Convert the rest of the coordinates to the appropriate type:
    coords_1[0], coords_1[1], coords_1[2] = int(coords_1[0]), int(coords_1[1]), float(coords_1[2])
    coords_1[3], coords_1[4], coords_1[5] = int(coords_1[3]), int(coords_1[4]), float(coords_1[5])
    coords_1[6], coords_1[8] =  float(coords_1[6]), float(coords_1[8])
    coords_1[10], coords_1[11], coords_1[12] = float(coords_1[10]), int(coords_1[11]), int(coords_1[12])
    
    print 'You have given the following coordinates:'
    print '*********************************'
    print 'Object : RA  --> %d %d %.2f' % ( coords_1[0], coords_1[1], coords_1[2] )
    print 'Object : Dec --> %d %d %.2f' % ( coords_1[3], coords_1[4], coords_1[5] )
    print '*********************************'
    print 'Beginning calculations...'
    
    # Convert the input strings to numbers
    latitude, direction_lat = coords_1[6], coords_1[7]
    longitude, direction_long = coords_1[8], coords_1[9]
    day, month, year = coords_1[10], coords_1[11], coords_1[12]
    
    # Convert coordinates to decimal form
    c_RA = hms2dh( coords_1[0], coords_1[1], coords_1[2] ) # in hours
    c_Dec = dms2dd( coords_1[3], coords_1[4], coords_1[5] )  # in degrees
    cr_Dec = radians(c_Dec) # in radians
    latitude_radians = radians(float(latitude))
    
    # ar and as correspond to the azimuth at rise and set respectively
    cos_ar =  sin(cr_Dec) / cos(latitude_radians)
    ar = acos(cos_ar) # in radians
    ar_degrees = degrees(ar) # in degrees
    as_degrees = 360.0 - ar_degrees
    ar_degrees_decimal = dd2dms(ar_degrees)
    as_degrees_decimal = dd2dms(as_degrees)
    
    hh = (1.0 / 15.0) * degrees( acos( -tan(latitude_radians) * tan(cr_Dec) ) ) # in hours
    
    LST_rise = 24.0 + c_RA - hh # in hours
    
    if (LST_rise > 24):
        LST_rise = LST_rise - 24.0
    
    LST_rise_hms = dh2hms(LST_rise)
    
    LST_set = c_RA + hh # in hours
    if (LST_set > 24):
        LST_set = LST_set - 24.0
    
    LST_set_hms = dh2hms(LST_set)
    
    # Convert values to GST and then to UT
    hrs_r, min_r, sec_r = LST_rise_hms[0:3]
    hrs_s, min_s, sec_s = LST_set_hms[0:3]
    GST_rise = lst2gst(hrs_r, min_r, sec_r, longitude, direction_long) # in hrs min sec
    GST_set =  lst2gst(hrs_s, min_s, sec_s, longitude, direction_long) # in hrs min sec
    UT_rise = gst2ut(GST_rise[0], GST_rise[1], GST_rise[2], day, month, year)
    UT_set =  gst2ut(GST_set[0], GST_set[1], GST_set[2], day, month, year)
    
    # These times are accurate to a few minutes.
    # For greater accuracy, we have to take into account atmospheric refraction.
    # It's effect is to increase the apparent altitude so that the times
    # of rising and setting will be earlier and later respectively than they
    # would have been without the atmosphere.
    
    psi = degrees( acos( sin(latitude_radians) / cos(cr_Dec) ) ) # in degrees
    
    # The magnitude of x, the refraction angle, at the horizon is about 
    #  34 minutes of arc. = 0.566667 degrees
    
    x = 0.566667 # in degrees
    delta_Alpha = degrees( asin( tan(radians(x)) / tan(radians(psi)) ) )
    
    # Calculate the apparent azimuths for rising and setting
    ar_ap = ar_degrees - delta_Alpha # in degrees
    as_ap = as_degrees + delta_Alpha # in degrees
    
    y = degrees(asin( sin(radians(x)) / sin(radians(psi)) ) ) # in degrees
    delta_time = (240.0 * y) / cos(cr_Dec) # in seconds
    
    # Convert to hours by dividing by 3600.0
    delta_time_hrs = delta_time / 3600.0
    
    # Find the apparent local sidereal times of rising and setting
    LST_rise_ap = LST_rise - delta_time_hrs # in hours
    LST_set_ap = LST_set + delta_time_hrs # in hours
    
    LST_rise_ap_hms = dh2hms(LST_rise_ap)
    LST_set_ap_hms = dh2hms(LST_set_ap)
    
    hrs_r_ap, min_r_ap, sec_r_ap = LST_rise_ap_hms[0:3]
    hrs_s_ap, min_s_ap, sec_s_ap = LST_set_ap_hms[0:3]
    
    # Convert values to GST and then to UT
    GST_rise_ap = lst2gst(hrs_r_ap, min_r_ap, sec_r_ap, longitude, direction_long) # in hrs min sec
    GST_set_ap = lst2gst(hrs_s_ap, min_s_ap, sec_s_ap, longitude, direction_long) # in hrs min sec
    UT_rise_ap = gst2ut(GST_rise_ap[0], GST_rise_ap[1], GST_rise_ap[2], day, month, year)
    UT_set_ap =  gst2ut(GST_set_ap[0], GST_set_ap[1], GST_set_ap[2], day, month, year)
    
    UT_all = UT_rise_ap + UT_set_ap
    print '*********************************' 
    print 'Calculations completed.'
    print '*********************************'
    print 'Results for %d-%d-%d' % (day,month,year)
    print '---OBJECT RISES AT---: %d:%d:%.2f UT' % (UT_all[0],UT_all[1],UT_all[2])
    print '---OBJECT SETS AT ---: %d:%d:%.2f UT' % (UT_all[3],UT_all[4],UT_all[5])
    return UT_all



####################################################################################
# Commandline run section
if __name__ == '__main__':
    if ( len(argv) != 14 ): 
        print "Error: Incorrect number of arguments. Need 13 input numbers: RA(target), Dec(target), latitude(observatory), longitude(observatory), date"
	print "Example calling sequence: python rise_set.py 23 39 20 21 42 00 30 0 64 1 24 8 1980"
	exit()
    
    try:
        print [argv[1],  argv[2], argv[3], argv[4],  argv[5], argv[6], argv[7], argv[8], argv[9], argv[10], argv[11], argv[12], argv[13] ]
	rise_set( [argv[1], argv[2], argv[3], argv[4],  argv[5], argv[6], argv[7], argv[8], argv[9], argv[10], argv[11], argv[12], argv[13] ] )
    except ValueError:
        print "Error: Incorrect number of arguments. Need 13 input numbers: RA(target), Dec(target), latitude(observatory), longitude(observatory), date"
	print "Example calling sequence: python rise_set.py 23 39 20 21 42 00 30 0 64 1 24 8 1980"
	exit()
