#!/usr/local/bin/python
#
# moon_pos
# Calculates the approximate position of the Moon for a given JD
# Input : date (JD)
# Output: moon_pos (RA, Dec, Distance) - in decimal hours, decimal degrees and
#  kilometers respectively
# Equations based on Astronomical Algorithms by Jean Meeus
#
# Example use 1; from the prompt: >python moon_pos.py 2455297.975
# Example use 2; from within a python program:
#    from moon_pos import moon_pos
#    position = moon_pos(2450667.958)
# Written by Y.Tsapras Feb 2009
# Last update by T.Tsapras 21 May 2010

# Import required libraries
from math import cos, sin, acos
from math import degrees as deg
from math import radians as rad
from rd_tables import rd_tables
from julday import julday
from dms2dd import dms2dd
from obliq_eclip import obliq_eclip
from ecl2equa import ecl2equa
from dd2dms import dd2dms
from dh2hms import dh2hms
from nutation import nutation
import sys

###############################################################################
# Function moon_pos starts here
def moon_pos(JD):
    [table1,size_table1] = rd_tables("table1.txt")
    [table2,size_table2] = rd_tables("table2.txt")
    
    T = (JD - 2451545.0) / 36525.0
    T2 = T * T
    T3 = T2 * T
    T4 = T3 * T
    
# Calculate the angles L1, D, M, M1, F
    
# The Moon's mean longitude (and including the constant 
#  term of the effect of light-time):
    L1 = (218.3164591 + 481267.88134236 * T - \
          0.0013268 * T2 + T3 / 538841.0 - T4 \
          / 65194000.0)
    
# To avoid working with large angles, reduce them to less
#  than 360 degrees
    while L1 > 360:
        L1 = L1 - 360
    
    while L1 < 0:
        L1 = L1 + 360
              
# Mean elongation of the Moon:
    D = (297.8502042 + 445267.1115168 * T - \
         0.0016300 * T2 + T3 / 545868.0 - T4 \
         / 113065000.0)
    
    while D > 360:
        D = D - 360
    
    while D < 0:
        D = D + 360
         
# Sun's mean anomaly:
    M = (357.5291092 + 35999.0502909 * T - \
         0.0001536 * T2 + T3 / 24490000.0)
    
    while M > 360:
        M = M - 360
    
    while M < 0:
        M = M + 360
    
# Moon's mean anomaly:
    M1 = (134.9634114 + 477198.8676313 * T + \
          0.0089970 * T2 + T3 / 69699.0 - \
          T4 / 14712000.0)
    
    while M1 > 360:
        M1 = M1 - 360
    
    while M1 < 0:
        M1 = M1 + 360
    
# Moon's argument of latitude (mean distance of the 
#  Moon from it's ascending node):
    F = (93.2720993 + 483202.01756273 * T - \
         0.0034029 * T2 - T3 / 3526000.0 + \
         T4 / 863310000.0)
    
    while F > 360:
        F = F - 360
    
    while F < 0:
        F = F + 360
    
# Three further arguments in degrees are needed:
    A1 = 119.75 + 131.849 * T
    A2 = 53.09 + 479264.290 * T
    A3 = 313.45 + 481266.484 * T
    
    while A1 > 360:
        A1 = A1 - 360
    
    while A1 < 0:
        A1 = A1 + 360
    
    while A2 > 360:
        A2 = A2 - 360
    
    while A2 < 0:
        A2 = A2 + 360
    
    while A3 > 360:
        A3 = A3 - 360
    
    while A3 < 0:
        A3 = A3 + 360
        
# Calculate correction needed due to the eccentricity
#  of the Earth's orbit around the Sun:
    E = 1.0 - 0.002516 * T - 0.0000074 * T2
    E2 = E * E
    
# Calculate the sums sig1, sigr and sigb of the tables
# The argument of each sine and cosine is a linear combination
# of the four fundamental arguments D, M, M1 and F (the first four
# terms in the tables). For example, the argument on the 8th line
# of table1 is 2*D -1*M -1*M1 and the contributions to sig1 and
# sigr are +57066 sin(2*D -1*M -1*M1) and -152138 cos(2*D -1*M -1*M1)
# respectively.
    
    sig1 = 0
    sigr = 0
    sigb = 0
    
    for elem in table1:
        Mcoeff =  int(elem[1])
	if Mcoeff == 1.0 or Mcoeff == -1.0:
	    Mcoeffok = Mcoeff * E
	elif Mcoeff == 2.0 or Mcoeff == -2.0:
	    Mcoeffok = Mcoeff * E2
	else:
	    Mcoeffok = Mcoeff	            
        term1 = (int(elem[0]) * D + Mcoeffok * M + \
                 int(elem[2]) * M1 + int(elem[3]) * F)
        sig1 = sig1 + int(elem[4]) * sin(rad(term1))
        sigr = sigr + int(elem[5]) * cos(rad(term1))
    
    for ele in table2:
        Mcoeff2 =  int(ele[1])
        if Mcoeff2 == 1.0 or Mcoeff2 == -1.0:
	    Mcoeff2ok = Mcoeff2 * E
        elif Mcoeff2 == 2.0 or Mcoeff2 == -2.0:
	    Mcoeff2ok = Mcoeff2 * E2
        else:
	    Mcoeff2ok = Mcoeff2		  
        term2 = (int(ele[0]) * D + Mcoeff2ok * M + \
                 int(ele[2]) * M1 + int(ele[3]) * F)
        sigb = sigb + int(ele[4]) * sin(rad(term2))
          
# Calculate extra additive terms due to the actions of
#  Venus and Jupiter:
# Additive to sig1:
    
    add_sig1 = (3958.0 * sin(rad(A1)) + 1962.0 * sin(rad(L1 - F)) + \
                318.0 * sin(rad(A2)))
    
# Additive to sigb:
    add_sigb = (-2235.0 * sin(rad(L1)) + 382.0 * sin(rad(A3)) + \
                175.0 * sin(rad(A1 - F)) + 175.0 * sin(rad(A1 + F)) + \
	        127.0 * sin(rad(L1 - M1)) - 115.0 * sin(rad(L1 + M1)))
    
    sig1 = sig1 + add_sig1
    sigb = sigb + add_sigb
    
# The coordinates for the Moon are:
    lamda = L1 + sig1 / 1000000.0 # in decimal degrees
    
    beta = sigb / 1000000.0 # in decimal degrees
    
    delta = 385000.56 + sigr / 1000.0 # in kilometers
    
# The apparent longitude of the Moon is obtained by adding to lamda
#  the nutation in longitude (delta_psi)
    [nutation_long, nutation_lat] = nutation(JD) # in arcsec
    nutation_long = nutation_long / 3600. # in degrees
    nutation_lat = nutation_lat / 3600. # in degrees
    
# Add nutation in longitude to lamda
    lamda = lamda + nutation_long
    
# Calculate obliquity
    obliquity = obliq_eclip(JD)
     
# Add nutation in latitude to obliquity
    obliquity = obliquity + nutation_lat     
    
# Convert to dms
    lamda_dms = dd2dms(lamda) # in dms
    beta_dms = dd2dms(beta) #in dms
    
# Calculate RA, Dec
    [right_asc,declin] = ecl2equa(lamda_dms[0],lamda_dms[1],lamda_dms[2], \
                                  beta_dms[0],beta_dms[1],beta_dms[2], \
				  obliquity)
# Bring RA to range 0-24
    while right_asc > 24:
        right_asc = right_asc - 12
    
    while right_asc < 0:
        right_asc = right_asc + 12
    
    print '***********************************************'
    print '* Moon RA: %.3f, Dec: %.3f' % (right_asc, declin)
    print '* Moon Distance: %.3f km' % delta
    print '***********************************************'
    return right_asc, declin, delta
    
# Allow calls from the prompt
# e.g python moon_pos.py 2455297.975
if __name__ == "__main__":
    JD = float(sys.argv[1])
    moon_pos(JD)
 
