#!/usr/bin/python
#
# rise_set
# Program to calculate the rise and set times of an object in the sky.
# It returns the times at which the altitude reaches 0 degrees.
# It does not account for parallax. (The times are accurate to within a few minutes)
# input: ([coordinates, latitude, date]) : ([RA, Dec, latitude, longitude, date]) 
#  : ([hrs, min, sec, deg, arcmin, arcsec, deg, direction, deg, direction, dd, mm, yyyy])
#   where latitude is a 2 element array and direction is 0 (North) or 1 (South)
#   and longitude is a 2 element array and direction is 0 (West) or 1 (East)
#   and date is in dd, mm, yyyy 
#  Total of 13 parameters
# output: rise and set times ([rise_hr, rise_min, rise_sec, set_hr, set_min, set_sec])
# Equations from Practical Astronomy with your Calculator by Peter Duffett-Smith
# Written by Y.Tsapras Feb 2009
# Modified by Y. Tsapras 18 May 2010

from math import cos, tan, sin, acos, pi, asin
from math import degrees as deg
from math import radians as rad
from hms2dh import hms2dh
from dh2hms import dh2hms
from dms2dd import dms2dd
from dd2dms import dd2dms
from gst2ut import gst2ut
from LST2GST import LST2GST

####################################################################################
# Function rise_set starts here
def rise_set(coords_1):
    print 'You have given the following coordinates:'
    print 'Object : RA --> %d %d %.2f' % (coords_1[0], coords_1[1], coords_1[2])
    print 'Object : Dec --> %d %d %.2f' % (coords_1[3], coords_1[4], coords_1[5])
    print '*********************************'
    print 'Beginning calculations...'
    latitude = coords_1[6]
    direction_lat = coords_1[7]
    longitude = coords_1[8]
    direction_long = coords_1[9]
    day = coords_1[10]
    month = coords_1[11]
    year = coords_1[12]
    
    # Convert coordinates to decimal form
    c_RA = hms2dh(coords_1[0], coords_1[1], coords_1[2]) # in hours
    c_Dec = dms2dd(coords_1[3], coords_1[4], coords_1[5])  # in degrees
    cr_Dec = rad(c_Dec) # in radians
    latitude_rad = rad(float(latitude))
    
    # ar and as correspond to the azimuth at rise and set respectively
    cos_ar =  sin(cr_Dec) / cos(latitude_rad)
    ar = acos(cos_ar) # in radians
    ar_deg = deg(ar) # in degrees
    as_deg = 360.0 - ar_deg
    ar_deg_decimal = dd2dms(ar_deg)
    as_deg_decimal = dd2dms(as_deg)
    
    hh = (1.0 / 15.0) * deg( acos( -tan(latitude_rad) * tan(cr_Dec) ) ) # in hours
    
    LST_rise = 24.0 + c_RA - hh # in hours
    
    if (LST_rise > 24):
        LST_rise = LST_rise - 24.0
    
    LST_rise_hms = dh2hms(LST_rise)
    
    LST_set = c_RA + hh # in hours
    if (LST_set > 24):
        LST_set = LST_set - 24.0
    
    LST_set_hms = dh2hms(LST_set)
    
    # Convert values to GST and then to UT
    hrs_r, min_r, sec_r = LST_rise_hms[0:3]
    hrs_s, min_s, sec_s = LST_set_hms[0:3]
    GST_rise = LST2GST(hrs_r, min_r, sec_r, longitude, direction_long) # in hrs min sec
    GST_set =  LST2GST(hrs_s, min_s, sec_s, longitude, direction_long) # in hrs min sec
    UT_rise = gst2ut(GST_rise[0], GST_rise[1], GST_rise[2], day, month, year)
    UT_set =  gst2ut(GST_set[0], GST_set[1], GST_set[2], day, month, year)
    
    # These times are accurate to a few minutes.
    # For greater accuracy, we have to take into account atmospheric refraction.
    # It's effect is to increase the apparent altitude so that the times
    # of rising and setting will be earlier and later respectively than they
    # would have been without the atmosphere.
    
    psi = deg( acos( sin(latitude_rad) / cos(cr_Dec) ) ) # in degrees
    
    # The magnitude of x, the refraction angle, at the horizon is about 
    #  34 minutes of arc. = 0.566667 degrees
    
    x = 0.566667 # in degrees
    delta_Alpha = deg( asin( tan(rad(x))/tan(rad(psi)) ) )
    
    # Calculate the apparent azimuths for rising and setting
    ar_ap = ar_deg - delta_Alpha # in degrees
    as_ap = as_deg + delta_Alpha # in degrees
    
    y = deg(asin( sin(rad(x))/sin(rad(psi)) )) # in degrees
    delta_time = (240.0 * y) / cos(cr_Dec) # in seconds
    
    # Convert to hours by dividing by 3600.0
    delta_time_hrs = delta_time / 3600.0
    
    # Find the apparent local sidereal times of rising and setting
    LST_rise_ap = LST_rise - delta_time_hrs # in hours
    LST_set_ap = LST_set + delta_time_hrs # in hours
    
    LST_rise_ap_hms = dh2hms(LST_rise_ap)
    LST_set_ap_hms = dh2hms(LST_set_ap)
    
    hrs_r_ap, min_r_ap, sec_r_ap = LST_rise_ap_hms[0:3]
    hrs_s_ap, min_s_ap, sec_s_ap = LST_set_ap_hms[0:3]
    
    # Convert values to GST and then to UT
    GST_rise_ap = LST2GST(hrs_r_ap, min_r_ap, sec_r_ap, longitude, direction_long) # in hrs min sec
    GST_set_ap = LST2GST(hrs_s_ap, min_s_ap, sec_s_ap, longitude, direction_long) # in hrs min sec
    UT_rise_ap = gst2ut(GST_rise_ap[0], GST_rise_ap[1], GST_rise_ap[2], day, month, year)
    UT_set_ap =  gst2ut(GST_set_ap[0], GST_set_ap[1], GST_set_ap[2], day, month, year)
    
    UT_all = UT_rise_ap + UT_set_ap
    print 'Calculations completed.'
    print '*********************************'
    print 'Results for %d-%d-%d' % (day,month,year)
    print 'Object rises at: %d:%d:%.2f UT' % (UT_all[0],UT_all[1],UT_all[2])
    print 'Object sets at: %d:%d:%.2f UT' % (UT_all[3],UT_all[4],UT_all[5])
    return UT_all
