#!/usr/bin/python
#
# sun_pos
# Program to calculate the apparent position of the Sun
# input: date (JD)
# output: sun_pos (RA, Dec) in decimal hours, decimal degrees
# Equations based on Astronomical Algorithms by Jean Meeus
#
# Example use 1; from the prompt: >python sun_pos.py 2455297.975
# Example use 2; from within a python program:
#    from sun_pos import sun_pos
#    position = sun_pos(2450667.958)
# Written by Y.Tsapras Feb 2009
# Last updated by Y.Tsapras Apr 2010

# Import required libraries
from math import cos,sin,asin,atan
from math import radians as rad
from math import degrees as deg
from julday import julday
from obliq_eclip import obliq_eclip
from dh2hms import dh2hms
from dd2dms import dd2dms
import sys

####################################################################################
# Function sun_pos starts here
def sun_pos(JD):
    T = (JD - 2451545.0) / 36525.0
    T2 = T * T
    T3 = T * T * T
    
# Mean equinox of the date (geometric mean longitude of Sun)
    L0 = 280.46645 + 36000.76983 * T + 0.0003032 * T2     # in degrees
    while L0 < 0:
        L0 = L0 + 360
    
    while L0 > 360:
        L0 = L0 - 360
    
# Mean anomaly of the Sun
    MA = (357.52910 + 35999.05030 * T - 0.0001559 * T2 - \
          0.00000048 * T3)       # in degrees
    while MA < 0:
        MA = MA + 360
    
    while MA > 360:
        MA = MA - 360
    	
# Eccentricity of the Earth's orbit
    epsilon = 0.016708617 - 0.000042037 * T - 0.0000001236 * T2 
    
# Sun's equation of centre
    C = ((1.914600 - 0.004817 * T - 0.000014 * T2) * sin(rad(MA)) + \
         (0.019993 - 0.000101 * T) * sin(rad(2 * MA)) + \
	 0.000290 * sin(rad(3 * MA)))    # in degrees
	 
# Then the Sun's true longitude is
    theta = L0 + C
    
# and it's true anomaly is
    v = MA + C
    
# The Sun's radius vector, or the distance from the Earth to
#  the Sun in AU is
    R = ((1.000001018 * (1 - epsilon * epsilon)) / (1 + epsilon * \
	 cos(rad(v))))	
    
# It is necessary to correct the true geometric longitude to the apparent
#  longitude (true equinox of the date)
    omega = 125.04 - 1934.136 * T    # in degrees
    lamda = theta - 0.00569 - 0.00478 * sin(rad(omega))
    
    obliquity = obliq_eclip(JD)    
    
# Then the RA, Dec of the apparent position of the Sun are
    obliq_corr = obliquity + 0.00256 * cos(rad(omega))
    sinDec = sin(rad(obliq_corr)) * sin(rad(lamda)) # eqn 24.7
    Dec = deg(asin(sinDec))
    nomRA = cos(rad(obliq_corr)) * sin(rad(lamda))
    RA = deg(atan(nomRA / cos(rad(lamda)))) 
    alpha = RA
    x = nomRA
    y = cos(rad(lamda))
    
# Removing the ambiguity which arises from taking the inverse tan.
# The rule is that alpha should lie in the appropriate quadrant:
# if x > 0 and y > 0 then alpha is between 0->90 degrees
# if x < 0 and y > 0 then alpha is between 90->180 degrees
# if x < 0 and y < 0 then alpha is between 180->270 degrees
# if x > 0 and y < 0 then alpha is between 270->360 degrees
# Add or subtract 180 or 360 to bring it in the correct range.
    corr_ang = 0
    if (x > 0 and y > 0):
        if (alpha > 180 and alpha < 270):
	    alpha = alpha - 180
	    corr_ang = -180
	elif (alpha < -90 and alpha > -180):
	    alpha = alpha + 180
	    corr_ang = 180
	else:
	    print 'alpha in correct range'
    elif (x < 0 and y > 0):
        if (alpha > 270 and alpha < 360):
	    alpha = alpha - 180
	    corr_ang = -180
	elif (alpha < 0 and alpha > -90):
	    alpha = alpha + 360
	    corr_ang = 360
	else:
	    print 'alpha in correct range'
    elif (x < 0 and y < 0):
        if (alpha > 0 and alpha < 90):
	    alpha = alpha + 180
	    corr_ang = 180
	else:
	    print 'alpha in correct range'
    elif (x > 0 and y < 0):
        if (alpha > 90 and alpha > 180):
	    alpha = alpha + 180
	    corr_ang = 180
	elif (alpha < -180 or alpha > -270):
	    alpha = alpha + 180 
	    corr_ang = 180 
	else:
	    print 'alpha in correct range'
    else:
        print 'Unknown range'
	
#    print 'x,y,corr_ang,alpha:',x,y,corr_ang,alpha
         
# Convert alpha to hours 
    alpha_hours = alpha / 15.0
    
# Bring to the range 0-24
    while alpha_hours > 24:
        alpha_hours = alpha_hours - 12
    while alpha_hours < 0:
        alpha_hours = alpha_hours + 12
    
# Convert from decimals    
    alpha_hms = dh2hms(alpha_hours) # in hms
    delta_dms = dd2dms(Dec) # in dms
    print '********************************************'       	
    print '* Sun RA: %.3f, Dec: %.3f' % (alpha_hours, Dec)
    print '********************************************'
    return alpha_hours, Dec
    
# Allow calls from the prompt
# e.g python sun_pos.py 2455297.975
if __name__ == "__main__":
    JD = float(sys.argv[1])
    sun_pos(JD)
