#!/usr/bin/python
#
# sun_pos
# Program to calculate the apparent position of the Sun
# input: Julian date (J2000.0)
# output: sun_pos (RA, Dec) in decimal hours, decimal degrees
#
# Equations from Astronomical Algorithms by Jean Meeus
#
# Written by Y.Tsapras Feb 2009
#
# Modification history:
# Y.Tsapras 21 May 2010
# Y.Tsapras 12 May 2011 - simplified calls and changed to use JD instead of dd, mm, yyyy 
# Y.Tsapras 12 May 2011 - added command line call
# Y.Tsapras 18 May 2011 - fixed a bug with alpha determined in the wrong quadrant
# Y.Tsapras 8 Feb 2012 - fixed a bug with alpha determined in the wrong quadrant
####################################################################################

# Import required libraries
from math import cos,sin,asin,atan
from math import radians, degrees
from julday import julday
from obliq_eclip import obliq_eclip
from dh2hms import dh2hms
from dd2dms import dd2dms
from sys import exit, argv
from caldat import caldat

####################################################################################
# Function sun_pos starts here
# calculates the apparent position of the Sun
def sun_pos( JD ):
    T = (JD - 2451545.0) / 36525.0
    T2 = T * T
    T3 = T * T * T
    
    # Mean equinox of the date (geometric mean longitude of Sun)
    L0 = 280.46645 + 36000.76983 * T + 0.0003032 * T2     # in degrees
    while L0 < 0:
        L0 = L0 + 360
    
    while L0 > 360:
        L0 = L0 - 360 
       
    # Mean anomaly of the Sun
    MA = (357.52910 + 35999.05030 * T - 0.0001559 * T2 - \
          0.00000048 * T3)       # in degrees
    while MA < 0:
        MA = MA + 360
    
    while MA > 360:
        MA = MA - 360
    
    # Eccentricity of the Earth's orbit
    epsilon = 0.016708617 - 0.000042037 * T - 0.0000001236 * T2 
    
    # Sun's equation of centre
    C = ( (1.914600 - 0.004817 * T - 0.000014 * T2) * sin( radians(MA) ) + \
          (0.019993 - 0.000101 * T) * sin( radians(2 * MA) ) + \
	  0.000290 * sin( radians(3 * MA) ) )    # in degrees
	 
    # Then the Sun's true longitude is
    theta = L0 + C
    
    # Convert theta to the standard equinox of J2000.0
    # Between the years 1900 and 2100, This can be performed with sufficient accuracy from 
    # theta = theta - 0.01397 * ( (caldat(JD)[-1]) - 2000.0 ) # See Meeus p.152
 
    # and it's true anomaly is
    v = MA + C

    # The Sun's radius vector, or the distance from the Earth to
    #  the Sun in AU is
    R = ( (1.000001018 * (1 - epsilon * epsilon) ) / (1 + epsilon * \
	 cos(radians(v)) ) )	

    # It is necessary to correct the true geometric longitude to the apparent
    #  longitude (true equinox of the date)
    omega = 125.04 - 1934.136 * T    # in degrees
    while omega < 0:
        omega = omega + 360
    
    while omega > 360:
        omega = omega - 360
    
    lamda = theta - 0.00569 - 0.00478 * sin(radians(omega))
       
    obliquity = obliq_eclip( JD )    

    # Then the RA, Dec of the apparent position of the Sun are
    obliq_corr = obliquity + 0.00256 * cos( radians(omega) )
    sinDec = sin( radians(obliq_corr) ) * sin( radians(lamda) )
    Dec = degrees( asin(sinDec) )
    nomRA = cos( radians(obliq_corr) ) * sin( radians(lamda) ) # nominator of the fraction
    RA = degrees( atan( nomRA / cos( radians(lamda) ) ) ) 
    alpha = RA
    x = nomRA
    y = cos( radians(lamda) )
    
    # Removing the ambiguity which arises from taking the inverse tan.
    # The rule is that alpha should lie in the appropriate quadrant:
    # alpha must be in the same quadrant as theta/lamda
    # Add or subtract 180 or 360 to bring it in the correct range.
    #
    #       /
    #   Q2  /  Q1
    #  ///////////
    #   Q3  /  Q4
    #       /
    #
    # Check it in the following order : Q1, Q2, Q3, Q4
    # The if statements define the quadrants and alpha is the angle
    # which must be brought to the correct quadrant
    if ( x > 0 and y > 0 ):
        if alpha > 180 and alpha < 270:
	    alpha = alpha - 180
	elif alpha < -90 and alpha > -180:
	    alpha = alpha + 180
	else:
	    print 'alpha in correct range'
    elif ( x < 0 and y > 0 ):
        if alpha > 270 and alpha < 360:
            alpha = alpha - 180
	elif alpha > -90 and alpha < 0 and lamda < 270:
            alpha = alpha + 180
	elif alpha > -90 and alpha < 0 and lamda > 270:
	    alpha = alpha + 360
	else:
            print 'alpha in correct range'
    elif ( x < 0 and y < 0 ):
        if alpha > 0 and alpha < 90:
            alpha = alpha + 180
	else:
            print 'alpha in correct range'
    elif ( x > 0 and y < 0 ):
        if alpha > 90 and alpha < 180:
            alpha = alpha + 180
	elif alpha < -180 and alpha > -270:
            alpha = alpha + 360 + 180
	elif alpha > -90 and alpha < 0:
	    if lamda > 90 and lamda < 180:
	        alpha = alpha + 180
	    else:
	        alpha = alpha + 360
	else:
            print 'alpha in correct range'
    else:
        print 'Unknown range'
             
    # Convert alpha to hours 
    alpha_hours = alpha / 15.0
    
    # Bring to range 0-24
    while alpha_hours > 24:
        alpha_hours = alpha_hours - 12
    while alpha_hours < 0:
        alpha_hours = alpha_hours + 12

    # Convert from decimals    
    alpha_hms = dh2hms(alpha_hours) # in hms
    delta_dms = dd2dms(Dec) # in dms
    
    print '*********************************' 
    print '--Sun-- RA: %.3f, Dec: %.3f' % (alpha_hours, Dec)
    print '*********************************'
    return alpha_hours, Dec


####################################################################################
# Commandline run section
if __name__ == '__main__':
    if ( len(argv) != 2 ): 
        print "Error: Incorrect number of arguments. Only the julian date is required."
	print "Example calling sequence: python sunpos.py 2455665.5"
	exit()
    
    try:
        sun_pos( float(argv[1]) )
    except ValueError:
        print "Input must be a number corresponding to a valid Julian Date."
	exit()
